close all
clear all
clc

% -- Loading data
[fname,pname] = uigetfile('*.mat','Test case selection');
data = load(strcat(pname,fname));
datavals = fieldnames(data);
disp(' ')
disp(data.casedescription)
disp(' ')
clear pname fname

% -- Filtering data 
lowestfrequency  = 15;    %Hz
highestfrequency = 1000;  %Hz
excludedata = [find(data.excfr<lowestfrequency);find(data.excfr>highestfrequency)];   
data.excfr(excludedata) = [];
for count = 1:size(datavals,1)
    isaccel = datavals{count};
    if strcmp(isaccel(1:3),'acc') || strcmp(isaccel(1:3),'for') || strcmp(isaccel(1:3),'tim')
        data.(datavals{count})(:,excludedata) = [];
    end
end
disp(['Frequency range (Hz)        : ' num2str(lowestfrequency) ' - ' num2str(highestfrequency)])
disp(['Number of measurements      : ' num2str(size(data.excfr,1))])
disp(['Number of accelerometers    : ' num2str(data.numberofaccelerometers)])
disp(['Excitation frequencies (Hz) : ' num2str(data.excfr')])
clear count excludedata isaccel lowestfrequency highestfrequency 

% -- Initialisation
acc = [];
vel = [];
dis = [];
forc = [];
coeffA = zeros(size(data.forcedata,2),data.numberofaccelerometers);
coeffB = zeros(size(data.forcedata,2),data.numberofaccelerometers);
coeffC = zeros(size(data.forcedata,2),1);
coeffD = zeros(size(data.forcedata,2),1);

% -- Calculating the maximum number of complete cycles
ncycles = min(floor(max(data.timedata)'.*data.excfr)); 
disp(['Max number of cycles avail. : ' num2str(ncycles)])

% -- Converting experimental data to single frequency harmonic function 
wb3 = waitbar(0,'Sine wave fitting');
for count = 1:size(data.forcedata,2)
    [hm,wh] = min(abs(data.timedata(:,count)-1/data.excfr(count)*ncycles));
    trigmatacc = [sin(data.excfr(count)*2*pi*data.timedata(1:wh,count)) cos(data.excfr(count)*2*pi*data.timedata(1:wh,count))];
    trigmatvel = (1/(data.excfr(count)*2*pi))*[-cos(data.excfr(count)*2*pi*data.timedata(1:wh,count)) sin(data.excfr(count)*2*pi*data.timedata(1:wh,count))];
    trigmatdis = -(1/(data.excfr(count)*2*pi).^2)*[sin(data.excfr(count)*2*pi*data.timedata(1:wh,count)) cos(data.excfr(count)*2*pi*data.timedata(1:wh,count))];
    count3 = 1;
    for count2 = 1:length(datavals)
        isaccel = datavals{count2};
        if strcmp(isaccel(1:3),'acc')
            coeff = trigmatacc\getfield(data,isaccel,{1:wh,count});
            acc = setfield(acc,['acc' num2str(count3) 'dataelab'],{1:wh,count},trigmatacc*coeff);
            vel = setfield(vel,['vel' num2str(count3) 'dataelab'],{1:wh,count},trigmatvel*coeff);
            dis = setfield(dis,['dis' num2str(count3) 'dataelab'],{1:wh,count},trigmatdis*coeff);
            coeffA(count,count3) = coeff(1);
            coeffB(count,count3) = coeff(2);
            count3 = count3+1;
        end
        if strcmp(isaccel(1:3),'for')
            coeff = trigmatacc\getfield(data,isaccel,{1:wh,count});
            forc = setfield(forc,['forc' num2str(count3) 'dataelab'],{1:wh,count},trigmatacc*coeff);
            coeffC(count,1) = coeff(1);
            coeffD(count,1) = coeff(2);
        end
        waitbar(((count*length(datavals))-(length(datavals)-count2))/(size(data.forcedata,2)*length(datavals)));
    end
end
close(wb3)
accvals = fieldnames(acc);
velvals = fieldnames(vel);
disvals = fieldnames(dis);
clear coeff count count2 count3 hm isaccel trigmatacc trigmatvel trigmatdis wb3 wh

% -- Plot sample acceleration measurement vs single frequency approximation

choice = menu('Do you want to plot real data vs approximation?','Yes','No');
if choice ==1
    numberofcolumns = 4;
    numbofperiods = 2;
    for idtoplot = 1:data.numberofaccelerometers
        figure('Name',['DOF ' num2str(idtoplot)])
        set(gca,'fontsize',14,'FontName','Timesnewroman')
        title('Acceleration vs single frequency approximation')
        for count=1:size(data.forcedata,2)
            subplot(ceil(size(data.forcedata,2)/numberofcolumns),numberofcolumns,count)
            plottime = data.timedata(1:round(size(acc.acc1dataelab,1)/ncycles*numbofperiods),count);
            plotreal = getfield(data,['acc' num2str(idtoplot) 'data'],{1:round(size(acc.acc1dataelab,1)/ncycles*numbofperiods),count});
            plotappr = getfield(acc,['acc' num2str(idtoplot) 'dataelab'],{1:round(size(acc.acc1dataelab,1)/ncycles*numbofperiods),count});
            hold on
            plot(plottime,plotreal,'r','LineWidth',0.6)
            plot(plottime,plotappr,'b','LineWidth',0.8)
            title([num2str(data.excfr(count)) ' Hz'])
            xlabel('Time (s)')
            ylabel('Acceleration m/s^2')
            line(xlim,[0,0],'LineStyle','-','LineWidth',0.2,'Color','k')
        end
    end
end
clear choice numberofcolumns numberofperiods idtoplot plottime plotreal plotappr

% --- Creating viscous damping configuration
dashpot = 0;
dashpot = dashpot+1;
L{dashpot} = zeros(data.numberofaccelerometers,data.numberofaccelerometers);
for count = 1:data.numberofaccelerometers-1
    L{dashpot}(count,count) = L{dashpot}(count,count)+1;
    L{dashpot}(count+1,count+1) = L{dashpot}(count+1,count+1)+1;
    L{dashpot}(count,count+1) = L{dashpot}(count,count+1)-1;
    L{dashpot}(count+1,count) = L{dashpot}(count+1,count)-1;
end
for count = 1:data.numberofaccelerometers
   dashpot = dashpot+1;
   L{dashpot} = zeros(data.numberofaccelerometers,data.numberofaccelerometers);
   L{dashpot}(count,count) = 1;
end
clear coeff count count2 count3 hm isaccel wh wb3

% --- Creating matrix of integrals and energy vector
matrixA = zeros(size(data.forcedata,2),dashpot);
wb3 = waitbar(0,'Creating matrix A');
for dcount = 1:dashpot
    for count = 1:size(data.forcedata,2)
        [hm,wh] = min(abs(data.timedata(:,count)-1/data.excfr(count)*ncycles));
        matrixA(count,dcount) = ncycles*pi/(data.excfr(count)*2*pi).^3*(coeffA(count,:)*L{dcount}*coeffA(count,:)'+coeffB(count,:)*L{dcount}*coeffB(count,:)');
        waitbar((dcount*size(data.forcedata,2)-size(data.forcedata,2)+count)/dashpot/size(data.forcedata,2))
    end
end
close(wb3)
ENFTOT = ncycles*(-coeffA(:,data.excitationdof).*coeffD+coeffB(:,data.excitationdof).*coeffC)*pi./((data.excfr*2*pi).^2);
clear hm wh wb3 dcount count

% --- Extracting damping parameters
damp = lsqnonneg(matrixA,ENFTOT);

% --- Constructing damping matrix
Cid = zeros(data.numberofaccelerometers,data.numberofaccelerometers);
for count = 1:dashpot
    Cid = Cid+L{count}*damp(count);
end
clear ENFTOT acc accvals coeffA coeffB coeffC coeffD count 
clear forc ncycles vel velvals datavals dis disvals

% --- Plot identified damping matrix 
figure
set(gca,'FontSize',14,'FontName','Timesnewroman')
h=bar3(Cid);
colorbar
colormap(flipud(hot))
for i = 1:length(h)
    zdata = ones(6*length(h),4);
    k = 1;
    for j = 0:6:(6*length(h)-6)
        zdata(j+1:j+6,:) = Cid(k,i);
        k = k+1;
    end
    set(h(i),'Cdata',zdata)
end
title(['Identified C - ' data.casedescription])
xlim([0 size(Cid,1)+1])
ylim([0 size(Cid,2)+1])
zlim([min(min(Cid)) max(max(Cid))*1.2])
zlabel('C (Ns/m)')
grid on

% --- Calculating energy % distribution
encountr = zeros(size(matrixA,2),1);
for count=1:size(matrixA,2)
    encountr(count) = sqrt(matrixA(:,count)'*matrixA(:,count)*damp(count)^2);
end
ratio=encountr/sum(encountr)*100;
for count=1:size(matrixA,2)
    ratio(count)=ratio(count)/sum(sum(abs(L{count})));
end
Eid = zeros(data.numberofaccelerometers,data.numberofaccelerometers);
for count = 1:dashpot
    Eid = Eid+abs(L{count})*ratio(count);
end

% --- Plot energy % distribution
figure
set(gca,'FontSize',14,'FontName','Timesnewroman')
h=bar3(Eid);
colorbar
colormap(flipud(bone))
for i = 1:length(h)
    zdata = ones(6*length(h),4);
    k = 1;
    for j = 0:6:(6*length(h)-6)
        zdata(j+1:j+6,:) = Eid(k,i);
        k = k+1;
    end
    set(h(i),'Cdata',zdata)
end
title(['Energy - ' data.casedescription])
xlim([0 size(Eid,1)+1])
ylim([0 size(Eid,2)+1])
zlim([0 100])
zlabel('Energy countribution (%)')
grid on

disp(' ')
disp('Identification performed, damping matrix saved as "Cid"')
disp(' ')

clear L Eid count damp dashpot data encountr h i j k matrixA ratio zdata
